@@ -35,6 +35,9 @@
 #define AKEY_CX85_DELETE           -29
 #define AKEY_CX85_YES              -30
 #define AKEY_TURBO                 -31
+#ifdef USE_UI_BASIC_ONSCREEN_KEYBOARD
+#define AKEY_KEYB                  -32
+#endif
 #ifdef DIRECTX
 	/* special menu directives */
 	#define AKEY32_MENU_SAVE_CONFIG     -107
