@@ -25,6 +25,7 @@
 #include <malloc.h>
 #include <string.h>
 #include <GLES/gl.h>
+#define GL_GLEXT_PROTOTYPES 1
 #include <GLES/glext.h>
 
 #include "atari.h"
@@ -32,6 +33,7 @@
 #include "colours.h"
 #include "akey.h"
 #include "cpu.h"
+#include "log.h"
 
 #include "androidinput.h"
 #include "graphics.h"
@@ -70,7 +72,12 @@ enum {
 };
 static GLuint texture[TEX_MAXNAMES];
 static UWORD conkey_vrt[CONK_VERT_MAX];
-static int conkey_lbl[CONK_VERT_MAX >> 2];
+static struct {
+	int x;
+	int y;
+	int w;
+	int h;
+} conkey_lbl[CONK_VERT_MAX / 8];
 static UWORD conkey_shadow[2 * 4];
 
 void Android_PaletteUpdate(void)
@@ -168,10 +175,22 @@ int Android_InitGraphics(void)
 		conkey_vrt[i    ] += tmp;
 		conkey_vrt[i + 1] += 4;
 	}
-	for (i = 0; i < CONK_VERT_MAX; i += 8) {
-		conkey_lbl[i >> 2] = conkey_vrt[i] + 6;
-		conkey_lbl[(i >> 2) + 1] = Android_ScreenH - (conkey_vrt[i + 1] - 1);
+
+	/* Determine location of console keys' labels. */
+	{
+		int key_w = conkey_vrt[2] - conkey_vrt[0];
+		int key_h = conkey_vrt[3] - conkey_vrt[5];
+		int key_h_slant = conkey_vrt[6] - conkey_vrt[0];
+		int label_w = key_w * key_h * 4 / (key_h_slant + key_h*4);
+		int label_h = label_w / 4;
+		for (i = 0; i < CONK_NUM; ++i) {
+			conkey_lbl[i].x = conkey_vrt[i*8 + 6];
+			conkey_lbl[i].y = Android_ScreenH - (conkey_vrt[i*8 + 7] + label_h + 1);
+			conkey_lbl[i].w = label_w;
+			conkey_lbl[i].h = label_h;
+		}
 	}
+
 	AndroidInput_ConOvl.keycoo = conkey_vrt;
 	AndroidInput_ConOvl.bbox.l = conkey_vrt[0];
 	AndroidInput_ConOvl.bbox.b = conkey_vrt[1];
@@ -388,9 +407,9 @@ ck:	if (AndroidInput_ConOvl.ovl_visible) {
 		glEnable(GL_TEXTURE_2D);	/* enable texturing */
 
 		glColor4f(1.0f, 1.0f, 1.0f, AndroidInput_ConOvl.opacity);
-		for (i = 0; i < CONK_VERT_MAX >> 2; i += 2) {
-			glTexParameteriv(GL_TEXTURE_2D, GL_TEXTURE_CROP_RECT_OES, crop_lbl[i >> 1]);
-			glDrawTexiOES(conkey_lbl[i], conkey_lbl[i + 1], 0, 40, 9);
+		for (i = 0; i < CONK_NUM; ++i) {
+			glTexParameteriv(GL_TEXTURE_2D, GL_TEXTURE_CROP_RECT_OES, crop_lbl[i]);
+			glDrawTexiOES(conkey_lbl[i].x, conkey_lbl[i].y, 0, conkey_lbl[i].w, conkey_lbl[i].h);
 		}
 		if (glGetError() != GL_NO_ERROR) Log_print("OpenGL error at console overlay");
 	}
@@ -452,6 +471,8 @@ void Update_Overlays(void)
 			c->statecnt = COVL_HOLD_TIME;
 		}
 		break;
+	default: /* COVL_HIDDEN */
+		;
 	}
 	if (c->hitkey == CONK_RESET) {
 		if (c->resetcnt >= RESET_HARD) {
