@@ -65,7 +65,6 @@
 #include "cfg.h"
 #include "cpu.h"
 #include "devices.h"
-#include "emuos.h"
 #include "esc.h"
 #include "gtia.h"
 #include "input.h"
@@ -174,8 +173,6 @@ int Atari800_auto_frameskip = FALSE;
 static double benchmark_start_time;
 #endif
 
-int emuos_mode = 1;	/* 0 = never use EmuOS, 1 = use EmuOS if real OS not available, 2 = always use EmuOS */
-
 #ifdef HAVE_SIGNAL
 volatile sig_atomic_t sigint_flag = FALSE;
 
@@ -291,48 +288,32 @@ int Atari800_LoadImage(const char *filename, UBYTE *buffer, int nbytes)
 	return TRUE;
 }
 
-static void copy_emuos(int machine_type) {
-	int rom_start_addr, padding;
-
-	rom_start_addr = Atari800_machine_type == Atari800_MACHINE_800 ? 0xd800 : 0xc000;
-	padding = 0x10000 - rom_start_addr - sizeof(emuos_h);
-	memset(MEMORY_os, 0, padding);
-	memcpy(MEMORY_os + padding, emuos_h, sizeof(emuos_h));
-}
-
 static int load_roms(void)
 {
-	if (Atari800_machine_type != Atari800_MACHINE_5200 && emuos_mode == 2) {
-		copy_emuos(Atari800_machine_type);
+	int basic_ver, xegame_ver;
+	SYSROM_ChooseROMs(Atari800_machine_type, MEMORY_ram_size, Atari800_tv_mode, &Atari800_os_version, &basic_ver, &xegame_ver);
+	if (Atari800_os_version == -1
+		|| !SYSROM_LoadImage(Atari800_os_version, MEMORY_os)) {
+		/* Missing OS ROM. */
 		Atari800_os_version = -1;
+		/* Avoid MEMORY_os containing old OS when the user explicitly removed
+		   all system ROMs from settings. */
+		memset(MEMORY_os, 0, sizeof(MEMORY_os));
+		return FALSE;
 	}
-	else {
-		int basic_ver, xegame_ver;
-		SYSROM_ChooseROMs(Atari800_machine_type, MEMORY_ram_size, Atari800_tv_mode, &Atari800_os_version, &basic_ver, &xegame_ver);
-		if (Atari800_os_version == -1
-		    || !Atari800_LoadImage(SYSROM_roms[Atari800_os_version].filename, MEMORY_os, SYSROM_roms[Atari800_os_version].size)) {
-			/* Missing OS ROM. */
-			Atari800_os_version = -1;
-			if (Atari800_machine_type != Atari800_MACHINE_5200 && emuos_mode == 1)
-				copy_emuos(Atari800_machine_type);
-			else
-				/* No OS ROM loaded. */
-				return FALSE;
-		}
-		else if (Atari800_machine_type != Atari800_MACHINE_5200) {
-			/* OS ROM found, try loading BASIC. */
-			MEMORY_have_basic = basic_ver != -1 && Atari800_LoadImage(SYSROM_roms[basic_ver].filename, MEMORY_basic, SYSROM_roms[basic_ver].size);
-			if (!MEMORY_have_basic)
-				/* Missing BASIC ROM. Don't fail when it happens. */
-				Atari800_builtin_basic = FALSE;
+	else if (Atari800_machine_type != Atari800_MACHINE_5200) {
+		/* OS ROM found, try loading BASIC. */
+		MEMORY_have_basic = basic_ver != -1 && SYSROM_LoadImage(basic_ver, MEMORY_basic);
+		if (!MEMORY_have_basic)
+			/* Missing BASIC ROM. Don't fail when it happens. */
+			Atari800_builtin_basic = FALSE;
 
-			if (Atari800_builtin_game) {
-				/* Try loading built-in XEGS game. */
-				if (xegame_ver == -1
-				    || !Atari800_LoadImage(SYSROM_roms[xegame_ver].filename, MEMORY_xegame, SYSROM_roms[xegame_ver].size))
-					/* Missing XEGS game ROM. */
-					Atari800_builtin_game = FALSE;
-			}
+		if (Atari800_builtin_game) {
+			/* Try loading built-in XEGS game. */
+			if (xegame_ver == -1
+				|| !SYSROM_LoadImage(xegame_ver, MEMORY_xegame))
+				/* Missing XEGS game ROM. */
+				Atari800_builtin_game = FALSE;
 		}
 	}
 
@@ -342,14 +323,14 @@ static int load_roms(void)
 
 int Atari800_InitialiseMachine(void)
 {
+	int have_roms;
 	ESC_ClearAll();
-	if (!load_roms())
-		return FALSE;
+	have_roms = load_roms();
 	Atari800_UpdateKeyboardDetached();
 	Atari800_UpdateJumper();
 	MEMORY_InitialiseMachine();
 	Devices_UpdatePatches();
-	return TRUE;
+	return have_roms;
 }
 
 /* Initialise any modules before loading the config file. */
@@ -561,8 +542,6 @@ int Atari800_Initialise(int *argc, char *argv[])
 			Atari800_tv_mode = Atari800_TV_PAL;
 		else if (strcmp(argv[i], "-ntsc") == 0)
 			Atari800_tv_mode = Atari800_TV_NTSC;
-		else if (strcmp(argv[i], "-emuos") == 0)
-			emuos_mode = 2;
 		else if (strcmp(argv[i], "-c") == 0) {
 			if (Atari800_machine_type == Atari800_MACHINE_800)
 				MEMORY_ram_size = 52;
@@ -1398,7 +1377,7 @@ void Atari800_StateRead(UBYTE version)
 		StateSav_ReadUBYTE(&temp, 1);
 		Atari800_SetTVMode(temp ? Atari800_TV_PAL : Atari800_TV_NTSC);
 		StateSav_ReadUBYTE(&temp, 1);
-		if (temp < 0 || temp >= Atari800_MACHINE_SIZE) {
+		if (temp >= Atari800_MACHINE_SIZE) {
 			temp = Atari800_MACHINE_XLXE;
 			Log_print("Warning: Bad machine type read in from state save, defaulting to XL/XE");
 		}
