@@ -9,6 +9,16 @@ extern FILE *BINLOAD_bin_file;
 int BINLOAD_Loader(const char *filename);
 extern int BINLOAD_start_binloading;
 extern int BINLOAD_loading_basic;
+
+/* Set to TRUE to enable loading of XEX with approximate disk speed */
+extern int BINLOAD_slow_xex_loading;
+
+/* Indicates that a DOS file is being currently slowly loaded. */
+extern int BINLOAD_wait_active;
+
+/* Set it to TRUE to pause the current loading of a DOS file. */
+extern int BINLOAD_pause_loading;
+
 #define BINLOAD_LOADING_BASIC_SAVED              1
 #define BINLOAD_LOADING_BASIC_LISTED             2
 #define BINLOAD_LOADING_BASIC_LISTED_ATARI       3
