@@ -86,12 +86,12 @@ static int fd_joystick1 = -1;
 #define MAX_JOYSTICKS	4
 static SDL_Joystick *joystick[MAX_JOYSTICKS] = { NULL, NULL, NULL, NULL };
 static int joystick_nbuttons[MAX_JOYSTICKS];
+static SDL_INPUT_RealJSConfig_t real_js_configs[MAX_JOYSTICKS];
 static int joysticks_found = 0;
-
-static int joystick0_use_hat = FALSE;
-static int joystick1_use_hat = FALSE;
-static int joystick0_use_hat_in_cfg_file = FALSE;
-static int joystick1_use_hat_in_cfg_file = FALSE;
+static struct js_state {
+	unsigned int port;
+	unsigned int trig;
+} sdl_js_state[MAX_JOYSTICKS];
 
 #define minjoy 10000			/* real joystick tolerancy */
 
@@ -144,11 +144,49 @@ static int SDLKeyBind(int *retval, char *sdlKeySymIntStr)
 	}
 }
 
+/*Set real joystick to use hat instead of axis*/
+static int set_real_js_use_hat(int joyIndex, const char* parm)
+{
+    real_js_configs[joyIndex].use_hat = Util_sscandec(parm) != 0 ? TRUE : FALSE;
+    return TRUE;
+}
+
+/*Reset configurations of the real joysticks*/
+static void reset_real_js_configs(void)
+{
+    int i;
+    for (i = 0; i < MAX_JOYSTICKS; i++) {
+        real_js_configs[i].use_hat = FALSE;
+    }
+}
+
+/*Write configurations of real joysticks*/
+static void write_real_js_configs(FILE* fp)
+{
+    int i;
+    for (i = 0; i < MAX_JOYSTICKS; i++) {
+        fprintf(fp, "SDL_JOY_%d_USE_HAT=%d\n", i, real_js_configs[i].use_hat);
+    }
+}
+
+/*Get pointer to a real joystick configuration*/
+SDL_INPUT_RealJSConfig_t* SDL_INPUT_GetRealJSConfig(int joyIndex)
+{
+    return &real_js_configs[joyIndex];
+}
+
 /* For getting sdl key map out of the config...
    Authors: B.Schreiber, A.Martinez
    cleaned up by joy */
 int SDL_INPUT_ReadConfig(char *option, char *parameters)
 {
+	static int was_config_initialized = FALSE;
+    
+	if (was_config_initialized == FALSE) {
+		reset_real_js_configs();
+		was_config_initialized=TRUE;
+	}
+    
 	if (strcmp(option, "SDL_JOY_0_ENABLED") == 0) {
 		PLATFORM_kbd_joy_0_enabled = (parameters != NULL && parameters[0] != '0');
 		return TRUE;
@@ -177,22 +215,15 @@ int SDL_INPUT_ReadConfig(char *option, char *parameters)
 		return SDLKeyBind(&KBD_STICK_1_UP, parameters);
 	else if (strcmp(option, "SDL_JOY_1_TRIGGER") == 0)
 		return SDLKeyBind(&KBD_TRIG_1, parameters);
-
-        else if (strcmp(option, "SDL_JOY_0_USE_HAT") == 0) {
-                if (strcmp(parameters,"1") == 0) {
-                    joystick0_use_hat = TRUE;
-                    joystick0_use_hat_in_cfg_file = TRUE;
-                }
-                return TRUE;
-        }
-        else if (strcmp(option, "SDL_JOY_1_USE_HAT") == 0) {
-                if (strcmp(parameters,"1") == 0) {
-                    joystick1_use_hat = TRUE;
-                    joystick1_use_hat_in_cfg_file = TRUE;
-                }
-                return TRUE;
-        }
-
+	else if (strcmp(option, "SDL_JOY_0_USE_HAT") == 0)
+		return set_real_js_use_hat(0,parameters);
+	else if (strcmp(option, "SDL_JOY_1_USE_HAT") == 0)
+		return set_real_js_use_hat(1,parameters);
+	else if (strcmp(option, "SDL_JOY_2_USE_HAT") == 0)
+		return set_real_js_use_hat(2,parameters);
+	else if (strcmp(option, "SDL_JOY_3_USE_HAT") == 0)
+		return set_real_js_use_hat(3,parameters);
+	else
 		return FALSE;
 }
 
@@ -215,9 +246,7 @@ void SDL_INPUT_WriteConfig(FILE *fp)
 	fprintf(fp, "SDL_JOY_1_DOWN=%d\n", KBD_STICK_1_DOWN);
 	fprintf(fp, "SDL_JOY_1_TRIGGER=%d\n", KBD_TRIG_1);
 
-        fprintf(fp, "SDL_JOY_0_USE_HAT=%d\n", joystick0_use_hat && (joystick0_use_hat_in_cfg_file == TRUE ));
-        fprintf(fp, "SDL_JOY_1_USE_HAT=%d\n", joystick1_use_hat && (joystick1_use_hat_in_cfg_file == TRUE ));
-
+	write_real_js_configs(fp);
 }
 
 void PLATFORM_SetJoystickKey(int joystick, int direction, int value)
@@ -1210,6 +1239,11 @@ int SDL_INPUT_Initialise(int *argc, char *argv[])
 	int no_joystick = FALSE;
 	int help_only = FALSE;
 
+	for(i = 0; i < MAX_JOYSTICKS; i++) {
+		sdl_js_state[i].port = INPUT_STICK_CENTRE;
+		sdl_js_state[i].trig = 0;
+	}
+
 	for (i = j = 1; i < *argc; i++) {
 #ifdef LPTJOY
 		int i_a = (i + 1 < *argc);		/* is argument available? */
@@ -1224,10 +1258,16 @@ int SDL_INPUT_Initialise(int *argc, char *argv[])
 		}
 
                 else if (strcmp(argv[i], "-joy0hat") == 0) {
-                        joystick0_use_hat = TRUE;
+                        real_js_configs[0].use_hat = TRUE;
                 }
                 else if (strcmp(argv[i], "-joy1hat") == 0) {
-                        joystick1_use_hat = TRUE;
+                        real_js_configs[1].use_hat = TRUE;
+                }
+                else if (strcmp(argv[i], "-joy2hat") == 0) {
+                        real_js_configs[2].use_hat = TRUE;
+                }
+                else if (strcmp(argv[i], "-joy3hat") == 0) {
+                        real_js_configs[3].use_hat = TRUE;
                 }
 #ifdef LPTJOY
 		else if (strcmp(argv[i], "-joy0") == 0) {
@@ -1249,6 +1289,8 @@ int SDL_INPUT_Initialise(int *argc, char *argv[])
 				Log_print("\t-nojoystick      Disable joystick");
                                 Log_print("\t-joy0hat         Use hat of joystick 0");
                                 Log_print("\t-joy1hat         Use hat of joystick 1");
+                                Log_print("\t-joy2hat         Use hat of joystick 1");
+                                Log_print("\t-joy3hat         Use hat of joystick 1");
 #ifdef LPTJOY
 				Log_print("\t-joy0 <pathname> Select LPTjoy0 device");
 				Log_print("\t-joy1 <pathname> Select LPTjoy1 device");
@@ -1349,26 +1391,26 @@ static int get_SDL_joystick_state(SDL_Joystick *joystick)
 
 static int get_SDL_joystick_hat_state(SDL_Joystick* joystick)
 {
-    Uint8 hat = SDL_JoystickGetHat(joystick, 0);
+	Uint8 hat = SDL_JoystickGetHat(joystick, 0);
 
-    if ( ((hat & SDL_HAT_LEFT)==SDL_HAT_LEFT) ) {
+	if ((hat & SDL_HAT_LEFT)==SDL_HAT_LEFT) {
+		if ((hat & SDL_HAT_LEFTDOWN)==SDL_HAT_LEFTDOWN) return INPUT_STICK_LL;
+		if ((hat & SDL_HAT_LEFTUP)==SDL_HAT_LEFTUP) return INPUT_STICK_UL;
+		return INPUT_STICK_LEFT;
+	}
+	else if ((hat & SDL_HAT_RIGHT)==SDL_HAT_RIGHT) {
+		if ((hat & SDL_HAT_RIGHTDOWN)==SDL_HAT_RIGHTDOWN) return INPUT_STICK_LR;
+		else if ((hat & SDL_HAT_RIGHTUP)==SDL_HAT_RIGHTUP) return INPUT_STICK_UR;
+		return INPUT_STICK_RIGHT;
+	}
+	else if ((hat & SDL_HAT_UP)==SDL_HAT_UP) {
+		return INPUT_STICK_FORWARD;
+	}
+	else if ((hat & SDL_HAT_DOWN)==SDL_HAT_DOWN) {
+		return INPUT_STICK_BACK;
+	}
 
-            if ( ((hat & SDL_HAT_LEFTDOWN)==SDL_HAT_LEFTDOWN) ) return INPUT_STICK_LL;
-            if ( ((hat & SDL_HAT_LEFTUP)==SDL_HAT_LEFTUP) ) return INPUT_STICK_UL;
-            return INPUT_STICK_LEFT;
-    }
-
-    else if ( ((hat & SDL_HAT_RIGHT)==SDL_HAT_RIGHT) ) {
-
-            if ( ((hat & SDL_HAT_RIGHTDOWN)==SDL_HAT_RIGHTDOWN) ) return INPUT_STICK_LR;
-            else if ( ((hat & SDL_HAT_RIGHTUP)==SDL_HAT_RIGHTUP) ) return INPUT_STICK_UR;
-            return INPUT_STICK_RIGHT;
-    }
-
-    else if ( ((hat & SDL_HAT_UP)==SDL_HAT_UP) ) return INPUT_STICK_FORWARD;
-    else if ( ((hat & SDL_HAT_DOWN)==SDL_HAT_DOWN) ) return INPUT_STICK_BACK;
-
-    return INPUT_STICK_CENTRE;
+	return INPUT_STICK_CENTRE;
 }
 
 static int get_LPT_joystick_state(int fd)
@@ -1417,12 +1459,6 @@ static int get_LPT_joystick_state(int fd)
 #endif /* LPTJOY */
 }
 
-
-static struct js_state {
-	unsigned int port;
-	unsigned int trig;
-} sdl_js_state[MAX_JOYSTICKS];
-
 static void update_SDL_joysticks(void)
 {
 	int joy;
@@ -1435,13 +1471,12 @@ static void update_SDL_joysticks(void)
 	for(joy = 0; joy < joysticks_found; joy++) {
 		int i;
 
-                if (joy == 0 && joystick0_use_hat == TRUE) {
-                        sdl_js_state[0].port = get_SDL_joystick_hat_state(joystick[0]);
-                } else if (joy == 1 && joystick1_use_hat == TRUE) {
-                        sdl_js_state[1].port = get_SDL_joystick_hat_state(joystick[1]);
-                } else {
-		sdl_js_state[joy].port = get_SDL_joystick_state(joystick[joy]);
-                }
+		if (real_js_configs[joy].use_hat == FALSE) {
+			sdl_js_state[joy].port = get_SDL_joystick_state(joystick[joy]);
+		}
+		else {
+			sdl_js_state[joy].port = get_SDL_joystick_hat_state(joystick[joy]);
+		}
 
 		sdl_js_state[joy].trig = 0;
 		for (i = 0; i < joystick_nbuttons[joy]; i++) {
