@@ -25,6 +25,7 @@
 #include <SDL.h>
 
 #include "af80.h"
+#include "bit3.h"
 #include "artifact.h"
 #include "atari.h"
 #include "colours.h"
@@ -89,7 +90,10 @@ void SDL_VIDEO_UpdatePaletteLookup(VIDEOMODE_MODE_t mode, int bpp_32)
 		PAL_BLENDING_UpdateLookup();
 	else
 #endif /* PAL_BLENDING */
-	if (mode != VIDEOMODE_MODE_NTSC_FILTER) {
+#ifdef NTSC_FILTER
+	if (mode != VIDEOMODE_MODE_NTSC_FILTER)
+#endif
+	{
 		void *dest;
 		if (bpp_32)
 			dest = SDL_PALETTE_buffer.bpp32;
@@ -101,9 +105,12 @@ void SDL_VIDEO_UpdatePaletteLookup(VIDEOMODE_MODE_t mode, int bpp_32)
 
 void PLATFORM_PaletteUpdate(void)
 {
+#ifdef NTSC_FILTER
 	if (SDL_VIDEO_current_display_mode == VIDEOMODE_MODE_NTSC_FILTER)
 		FILTER_NTSC_Update(FILTER_NTSC_emu);
-	else {
+	else
+#endif
+	{
 #if HAVE_OPENGL
 		if (SDL_VIDEO_opengl)
 			SDL_VIDEO_GL_PaletteUpdate();
@@ -133,6 +140,7 @@ void PLATFORM_MapRGB(void *dest, int const *palette, int size)
 		SDL_VIDEO_SW_MapRGB(dest, palette, size);
 }
 
+#ifdef NTSC_FILTER
 static void UpdateNtscFilter(VIDEOMODE_MODE_t mode)
 {
 	if (mode != VIDEOMODE_MODE_NTSC_FILTER && FILTER_NTSC_emu != NULL) {
@@ -152,6 +160,7 @@ static void UpdateNtscFilter(VIDEOMODE_MODE_t mode)
 		FILTER_NTSC_Update(FILTER_NTSC_emu);
 	}
 }
+#endif
 
 void PLATFORM_SetVideoMode(VIDEOMODE_resolution_t const *res, int windowed, VIDEOMODE_MODE_t mode, int rotate90)
 {
@@ -207,7 +216,9 @@ void PLATFORM_SetVideoMode(VIDEOMODE_resolution_t const *res, int windowed, VIDE
 	SDL_VIDEO_SW_SetVideoMode(res, windowed, mode, rotate90);
 #endif
 	SDL_VIDEO_current_display_mode = mode;
+#ifdef NTSC_FILTER
 	UpdateNtscFilter(mode);
+#endif
 	PLATFORM_DisplayScreen();
 
 	/* For unknown reason (maybe window manager-related), when SDL_SetVideoMode
@@ -235,12 +246,19 @@ VIDEOMODE_resolution_t *PLATFORM_AvailableResolutions(unsigned int *size)
 	VIDEOMODE_resolution_t *resolutions;
 	unsigned int num_modes;
 	unsigned int i;
-	if (modes == (SDL_Rect**)0 || modes == (SDL_Rect**)-1)
+	if (modes == (SDL_Rect**)0)
 		return NULL;
+	if (modes == (SDL_Rect**)-1) {
+		resolutions = (VIDEOMODE_resolution_t *)Util_malloc(sizeof(VIDEOMODE_resolution_t));
+		resolutions[0].width = 336;
+		resolutions[0].height = 240;
+		*size = 1;
+		return resolutions;
+	}
 	/* Determine number of available modes. */
 	for (num_modes = 0; modes[num_modes] != NULL; ++num_modes);
 
-	resolutions = Util_malloc(num_modes * sizeof(VIDEOMODE_resolution_t));
+	resolutions = (VIDEOMODE_resolution_t *)Util_malloc(num_modes * sizeof(VIDEOMODE_resolution_t));
 	for (i = 0; i < num_modes; i++) {
 		resolutions[i].width = modes[i]->w;
 		resolutions[i].height = modes[i]->h;
@@ -334,8 +352,15 @@ void SDL_VIDEO_InitSDL(void)
 	/* Get the desktop resolution */
 	{
 		SDL_VideoInfo const * const info = SDL_GetVideoInfo();
-		desktop_resolution.width = info->current_w;
-		desktop_resolution.height = info->current_h;
+		if (info->current_w == 0 || info->current_h == 0) {
+			/* fix for Atari's XBIOS SDL backend */
+			desktop_resolution.width = 320;
+			desktop_resolution.height = 240;
+		} else {
+			desktop_resolution.width = info->current_w;
+			desktop_resolution.height = info->current_h;
+		}
+
 		SDL_VIDEO_native_bpp = info->vfmt->BitsPerPixel;
 	}
 
@@ -449,7 +474,10 @@ int SDL_VIDEO_Initialise(int *argc, char *argv[])
 void SDL_VIDEO_Exit(void)
 {
 	SDL_VIDEO_QuitSDL();
-	if (FILTER_NTSC_emu) {
+#ifdef NTSC_FILTER
+	if (FILTER_NTSC_emu)
+#endif
+	{
 		/* Turning filter off */
 		FILTER_NTSC_Delete(FILTER_NTSC_emu);
 		FILTER_NTSC_emu = NULL;
@@ -481,12 +509,21 @@ void SDL_VIDEO_BlitNormal16(Uint32 *dest, Uint8 *src, int pitch, int width, int
 	while (height > 0) {
 		pos = width_32;
 		do {
+#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+			pos--;
+			c = src[pos];
+			quad = palette16[c];
+			pos--;
+			c = src[pos];
+			quad += palette16[c] << 16;
+#else
 			pos--;
 			c = src[pos];
 			quad = palette16[c] << 16;
 			pos--;
 			c = src[pos];
 			quad += palette16[c];
+#endif
 			start32[pos >> 1] = quad;
 		} while (pos > 0);
 		src += Screen_WIDTH;
@@ -540,12 +577,21 @@ void SDL_VIDEO_BlitXEP80_16(Uint32 *dest, Uint8 *src, int pitch, int width, int
 	while (height > 0) {
 		pos = width_32;
 		do {
+#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+			pos--;
+			c = src[pos];
+			quad = palette16[c];
+			pos--;
+			c = src[pos];
+			quad += palette16[c] << 16;
+#else
 			pos--;
 			c = src[pos];
 			quad = palette16[c] << 16;
 			pos--;
 			c = src[pos];
 			quad += palette16[c];
+#endif
 			start32[pos >> 1] = quad;
 		} while (pos > 0);
 		src += XEP80_SCRN_WIDTH;
@@ -586,6 +632,18 @@ void SDL_VIDEO_BlitProto80_8(Uint32 *dest, int first_column, int last_column, in
 			int i;
 			pixels = PBI_PROTO80_GetPixels(first_line, column);
 			for (i = 0; i < 2; i++) {
+#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+				if (pixels & 0x10)
+					quad = 0x0000000f;
+				else
+					quad = 0x00000000;
+				if (pixels & 0x20)
+					quad |= 0x00000f00;
+				if (pixels & 0x40)
+					quad |= 0x000f0000;
+				if (pixels & 0x80)
+					quad |= 0x0f000000;
+#else
 				if (pixels & 0x80)
 					quad = 0x0000000f;
 				else
@@ -596,6 +654,7 @@ void SDL_VIDEO_BlitProto80_8(Uint32 *dest, int first_column, int last_column, in
 					quad |= 0x000f0000;
 				if (pixels & 0x10)
 					quad |= 0x0f000000;
+#endif
 				*start32++ = quad;
 				pixels <<= 4;
 			}
@@ -620,6 +679,16 @@ void SDL_VIDEO_BlitProto80_16(Uint32 *dest, int first_column, int last_column, i
 			int i;
 			pixels = PBI_PROTO80_GetPixels(first_line, column);
 			for (i = 0; i < 4; i++) {
+#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+				if (pixels & 0x40)
+					quad = white;
+				else
+					quad = black;
+				if (pixels & 0x80)
+					quad |= white2;
+				else
+					quad |= black2;
+#else
 				if (pixels & 0x80)
 					quad = white;
 				else
@@ -628,6 +697,7 @@ void SDL_VIDEO_BlitProto80_16(Uint32 *dest, int first_column, int last_column, i
 					quad |= white2;
 				else
 					quad |= black2;
+#endif
 				*start32++ = quad;
 				pixels <<= 2;
 			}
@@ -660,6 +730,7 @@ void SDL_VIDEO_BlitProto80_32(Uint32 *dest, int first_column, int last_column, i
 	}
 }
 
+#ifdef AF80
 void SDL_VIDEO_BlitAF80_8(Uint32 *dest, int first_column, int last_column, int pitch, int first_line, int last_line, int blink)
 {
 	int skip = pitch - (last_column - first_column)*2;
@@ -673,6 +744,18 @@ void SDL_VIDEO_BlitAF80_8(Uint32 *dest, int first_column, int last_column, int p
 			int colour;
 			pixels = AF80_GetPixels(first_line, column, &colour, blink);
 			for (i = 0; i < 2; i++) {
+#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+				if (pixels & 0x01)
+					quad = colour << 24;
+				else
+					quad = 0;
+				if (pixels & 0x02)
+					quad |= colour << 16;
+				if (pixels & 0x04)
+					quad |= colour << 8;
+				if (pixels & 0x08)
+					quad |= colour;
+#else
 				if (pixels & 0x01)
 					quad = colour;
 				else
@@ -683,6 +766,7 @@ void SDL_VIDEO_BlitAF80_8(Uint32 *dest, int first_column, int last_column, int p
 					quad |= colour << 16;
 				if (pixels & 0x08)
 					quad |= colour << 24;
+#endif
 				*start32++ = quad;
 				pixels >>= 4;
 			}
@@ -706,6 +790,16 @@ void SDL_VIDEO_BlitAF80_16(Uint32 *dest, int first_column, int last_column, int
 			int colour;
 			pixels = AF80_GetPixels(first_line, column, &colour, blink);
 			for (i = 0; i < 4; i++) {
+#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+				if (pixels & 0x01)
+					quad = palette16[colour] << 16;
+				else
+					quad = black2;
+				if (pixels & 0x02)
+					quad |= palette16[colour];
+				else
+					quad |= black;
+#else
 				if (pixels & 0x01)
 					quad = palette16[colour];
 				else
@@ -714,6 +808,7 @@ void SDL_VIDEO_BlitAF80_16(Uint32 *dest, int first_column, int last_column, int
 					quad |= palette16[colour] << 16;
 				else
 					quad |= black2;
+#endif
 				*start32++ = quad;
 				pixels >>= 2;
 			}
@@ -745,6 +840,119 @@ void SDL_VIDEO_BlitAF80_32(Uint32 *dest, int first_column, int last_column, int
 		start32 += skip;
 	}
 }
+#endif /* AF80 */
+
+#ifdef BIT3
+void SDL_VIDEO_BlitBIT3_8(Uint32 *dest, int first_column, int last_column, int pitch, int first_line, int last_line, int blink)
+{
+	int skip = pitch - (last_column - first_column)*2;
+	register Uint32 *start32 = dest;
+	unsigned int column;
+	UBYTE pixels;
+	register Uint32 quad;
+	for (; first_line < last_line; first_line++) {
+		for (column = first_column; column < last_column; column++) {
+			int i;
+			int colour;
+			pixels = BIT3_GetPixels(first_line, column, &colour, blink);
+			for (i = 0; i < 2; i++) {
+#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+				if (pixels & 0x01)
+					quad = colour << 24;
+				else
+					quad = 0;
+				if (pixels & 0x02)
+					quad |= colour << 16;
+				if (pixels & 0x04)
+					quad |= colour << 8;
+				if (pixels & 0x08)
+					quad |= colour;
+#else
+				if (pixels & 0x01)
+					quad = colour;
+				else
+					quad = 0;
+				if (pixels & 0x02)
+					quad |= colour << 8;
+				if (pixels & 0x04)
+					quad |= colour << 16;
+				if (pixels & 0x08)
+					quad |= colour << 24;
+#endif
+				*start32++ = quad;
+				pixels >>= 4;
+			}
+		}
+		start32 += skip;
+	}
+}
+
+void SDL_VIDEO_BlitBIT3_16(Uint32 *dest, int first_column, int last_column, int pitch, int first_line, int last_line, int blink, Uint16 *palette16)
+{
+	Uint32 const black = (Uint32)palette16[0];
+	Uint32 const black2 = black << 16;
+	int skip = pitch - (last_column - first_column)*4;
+	register Uint32 *start32 = dest;
+	unsigned int column;
+	UBYTE pixels;
+	register Uint32 quad;
+	for (; first_line < last_line; first_line++) {
+		for (column = first_column; column < last_column; column++) {
+			int i;
+			int colour;
+			pixels = BIT3_GetPixels(first_line, column, &colour, blink);
+			for (i = 0; i < 4; i++) {
+#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+				if (pixels & 0x01)
+					quad = palette16[colour] << 16;
+				else
+					quad = black2;
+				if (pixels & 0x02)
+					quad |= palette16[colour];
+				else
+					quad |= black;
+#else
+				if (pixels & 0x01)
+					quad = palette16[colour];
+				else
+					quad = black;
+				if (pixels & 0x02)
+					quad |= palette16[colour] << 16;
+				else
+					quad |= black2;
+#endif
+				*start32++ = quad;
+				pixels >>= 2;
+			}
+		}
+		start32 += skip;
+	}
+}
+
+void SDL_VIDEO_BlitBIT3_32(Uint32 *dest, int first_column, int last_column, int pitch, int first_line, int last_line, int blink, Uint32 *palette32)
+{
+	Uint32 const black = palette32[0];
+	int skip = pitch - (last_column - first_column)*8;
+	register Uint32 *start32 = dest;
+	unsigned int column;
+	UBYTE pixels;
+	for (; first_line < last_line; first_line++) {
+		for (column = first_column; column < last_column; column++) {
+			int i;
+			int colour;
+			pixels = BIT3_GetPixels(first_line, column, &colour, blink);
+			for (i = 0; i < 8; i++) {
+				if (pixels & 0x01)
+					*start32++ = palette32[colour];
+				else
+					*start32++ = black;
+				pixels >>= 1;
+			}
+		}
+		start32 += skip;
+	}
+}
+#endif /* BIT3 */
 
 void SDL_VIDEO_SetScanlinesPercentage(int value)
 {
