@@ -1,6 +1,7 @@
 #ifndef SDL_VIDEO_SW_H_
 #define SDL_VIDEO_SW_H_
 
+#include "platform.h"
 #include "videomode.h"
 
 void SDL_VIDEO_SW_DisplayScreen(void);
@@ -14,6 +15,13 @@ extern int SDL_VIDEO_SW_bpp;
 int SDL_VIDEO_SW_SetBpp(int value);
 int SDL_VIDEO_SW_ToggleBpp(void);
 
+/* Returns parameters of the current display pixel format. Used when computing
+   lookup tables used for blitting the Atari screen to display surface. */
+void SDL_VIDEO_SW_GetPixelFormat(PLATFORM_pixel_format_t *format);
+/* Convert a table of RGB values, PALETTE, of size SIZE, to a display's native
+   format and store it in the lookup table DEST. */
+void SDL_VIDEO_SW_MapRGB(void *dest, int const *palette, int size);
+
 /* Read/write to configuration file. */
 int SDL_VIDEO_SW_ReadConfig(char *option, char *parameters);
 void SDL_VIDEO_SW_WriteConfig(FILE *fp);
